/**
 * Author: Ash Matheson
 * Date: Aug 15, 1999
 * Comments:
 * OK.  This is the main driver for the resolution switching engine.
 * It's pretty simple.  Here's how it works. Upon initialization of the class,
 * all the available video modes are stored in the class as a STL list of DEVMODES.
 * A call to ChangeRes will then change the resolution to the requested resolution
 * (if available).
 * 
 * A lot of the logic and code for the resolution switching code was based on the
 * work done by Ryan Haksi.  I haven't been able to get a hold of him, but his last
 * known Web page was http://home.bc.rogers.wave.ca/borealis/opengl.html
 * If anyone has any current contact info for Ryan, I'd appreciate the pass-along.
 * 
 */

#include "stdafx.h"
#include "change.h"
#include "logging.h"

/**
 *	Class:	ResManager
 *	Method: ResManager()
 *	Desc:	Default Constructor
 */
ResManager::ResManager()
{
  Init();
}

/**
 *	Class:	ResManager
 *	Method: ResManager(int width, int height, int depth)
 *	Desc:	Paramaterized Constructor
 */
ResManager::ResManager(int width, int height, int depth)
{
	Init();
	ChangeRes(width, height, depth);
}

/**
 *	Class:	ResManager
 *	Method: ~ResManager()
 *	Desc:	Default Constructor
 */
ResManager::~ResManager()
{
	// clean out the list of DeviceModes
	if (m_stdDeviceModes.size() > 0)
	{
		m_stdDeviceModes.erase(m_stdDeviceModes.begin(), m_stdDeviceModes.end());
	}
}


ResManager::Init()
{
	int  nModeExist;
	DEVMODE devMode;
	devMode.dmSize = sizeof(DEVMODE);

	// Get the current list of Available DevModes and store them into 
	for (int i=0; ;i++) 
	{
		nModeExist = EnumDisplaySettings(NULL, i, &devMode);

		if (nModeExist != 1) 
		{
			// End of modes.  bail out.
			break;
		}
		else
		{
			// Add the driver to the list.
			m_stdDeviceModes.push_front(devMode);
		}
	}
}


/**
 *	Class:	ResManager
 *	Method: ChangeRes(int width, int height, int depth)
 *	Desc:	Change the resolution to one of the available video modes.
 */
void ResManager::ChangeRes(int width, int height, int depth)
{
	bool success = false;
	int nModeSwitch;
	std::list<DEVMODE>::iterator resIter;

	// Iterate through the DeviceModes list, looking for a match
	// if one is found, switch to that resolution and bail.
	// Otherwise, leave alone.
	resIter = m_stdDeviceModes.begin();

	while (resIter != m_stdDeviceModes.end())
	{
		if ( ((*resIter).dmBitsPerPel == depth)
			   &&((*resIter).dmPelsWidth == width) 
			   &&((*resIter).dmPelsHeight == height))
		{
			nModeSwitch = ChangeDisplaySettings(&(*resIter), CDS_FULLSCREEN); 
			if (nModeSwitch==DISP_CHANGE_SUCCESSFUL) break;

			// Whoops, it didn't work.  Possibly running Win95?  Try without switching resolutions
			int nCloseMode = 0;
			EnumDisplaySettings(NULL, nCloseMode, &(*resIter));
			(*resIter).dmBitsPerPel = depth; 
			(*resIter).dmPelsWidth  = width; 
			(*resIter).dmPelsHeight = height; 
			(*resIter).dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT; 

			nModeSwitch = ChangeDisplaySettings(&(*resIter), CDS_FULLSCREEN); 
			if(nModeSwitch == DISP_CHANGE_SUCCESSFUL) 
			{
				success = true;
				break;
			}

			// Nope, what else can we try?  Separately change the BitDepth and then resolution
			(*resIter).dmFields = DM_BITSPERPEL; 
			nModeSwitch = ChangeDisplaySettings(&(*resIter), CDS_FULLSCREEN);
			if(nModeSwitch == DISP_CHANGE_SUCCESSFUL) 
			{
				(*resIter).dmFields = DM_PELSWIDTH | DM_PELSHEIGHT; 
				nModeSwitch = ChangeDisplaySettings(&(*resIter), CDS_FULLSCREEN); 
				if(nModeSwitch == DISP_CHANGE_SUCCESSFUL) 
				{
					success = true;
					break;
				}
				ChangeDisplaySettings(NULL, 0);
				break;
			}
		}
		resIter++;
	}

	// Did we change the resolution?
	if (true == success)
	{
		m_Current = (*resIter);
	}
	else
	{
		// Error and list the available Video modes ...
		ListResolutions();
	}
}

/**
 *	Class:	ResManager
 *	Method: RestoreResolution()
 *	Desc:	Restores to the original Video resolution.
 */
void ResManager::RestoreResolution()
{
	// Pretty Simple ... Like me.
	ChangeDisplaySettings(NULL, 0);
}

// Utility Methods ----------------------------------------

/**
 *	Class:	ResManager
 *	Method: RestoreResolution()
 *	Desc:	Restores to the original Video resolution.
 */
void ResManager::ListResolutions()
{
	char buf[255];
	std::list<DEVMODE>::iterator resIter;

	// Iterate through the DeviceModes list, Displaying a summary of each
	resIter = m_stdDeviceModes.begin();

	sprintf(buf, "The following Resolutions are available:");
	Log(LOG_MESSAGE, buf);

	while (resIter != m_stdDeviceModes.end())
	{
		sprintf(buf,"Width: %d, Height:%d, BPP: %d",(*resIter).dmBitsPerPel, (*resIter).dmPelsWidth, (*resIter).dmPelsHeight);
		Log(LOG_MESSAGE, buf);
		resIter++;
	}

}